--******************************************************************************************************************************
--
-- Description: Model 377 product feature description.
--				Note: This package is common to all SigFPGA products, but tailored to the Model 377.
--					
-- Rev History: 07-23-2014:	Inital release.
--
--******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

package model_specific_package is

	function zero_to_one (ch_cnt : integer) return integer;
	
	-- Assign default values specific to this model number.
	constant MODEL			: std_logic_vector(11 downto 0) := x"377";		-- Model number.
	constant REVISION		: std_logic_vector(31 downto 0) := x"07232014";	-- FPGA design revision.
	constant GPIO_COUNT		: integer := 6;  		-- Number of GPIO pins.
	constant USER_COUNT		: integer := 62;  		-- Number of user pins.
	constant PCI_LANES		: integer := 8;  		-- Number of PCIe electrical lanes.
	constant AXI_WIDTH		: integer := 128;  		-- Width of AXI bus connecting the EZDMA2 core to the Xilinx endpoint core.
	constant QDR_ADDR_WIDTH	: integer := 21;  		-- Width of QDR II+ SRAM address.
	constant BPI_ADD		: integer := 23;		-- Size of BPI configuration flash address bus.
	
	-- Define the transmitter channel characteristics.
	constant TX_CHANNEL_COUNT 	: integer := 0;		-- No TX channels on this model.
	constant TX_DATA_DDR		: boolean := false;	-- Not applicable to this model.
	constant TX_CLK_FWD			: boolean := false;	-- Not applicable to this model.
	constant TX_ITEM_SIZE 		: integer := 1;		-- Not applicable to this model.
	constant TX_ITEM_STUFF 		: integer := 1;		-- Must be greater than zero.
	constant TX_CLK_SELECT 		: integer := 0;		-- Not applicable to this model.

	-- Define the receiver channel characteristics.
	constant RX_CHANNEL_COUNT 	: integer := 4;		-- Four RX channels on this model.
	constant RX_ITEM_SIZE 		: integer := 16;	-- 16-bit ADC
	constant RX_ITEM_PER_CLK	: integer := 1;		-- Number of items generated per clock edge.
	constant RX_ITEM_STUFF 		: integer := 1;		-- Number of items stuffed into the ADC output bus.
	constant RX_INTERLEAVE 		: integer := 2;		-- Two bits are interleaved across DDR clock edges.
	constant RX_CHAN_PER_OVR	: integer := 1;		-- Number of channels assigned to each over-range bit.
	constant RX_CLK_SELECT 		: integer := 0;		-- Route sample clock to the RX channel.
	
	-- Set the number of active DMA channels.
	constant DMA_CHANNEL_COUNT 	: integer := TX_CHANNEL_COUNT+RX_CHANNEL_COUNT;		-- Match the number of RX/TX channels.
	
	-- The following values are hardware specific to the model number ADC interface.
	type ADC_IO_DELAY is array (0 to (RX_CHANNEL_COUNT*RX_ITEM_SIZE/RX_INTERLEAVE)-1) of integer;
	-- Set the ADC data and over-range IOB initial delay values.
	-- Note:  The position of each initialization value will match the array index, so the zero index is on the left.
	constant ADC_DATA_INIT		: ADC_IO_DELAY := (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
	constant ADC_OVR_INIT		: integer := 0;
	-- Push the data bits away from the clock edge (78.125 ps per count).
	-- Note:  Set to zero if the clock edge is centered in the data valid window.
	constant ADC_CLK_INIT		: integer := 0;
	-- Command the number of ADC clock forward sources to evaluate.
	constant ADC_CLK_SOURCES	: integer := 2;
	-- Command the clock lock function to search for the rising and falling edge of the forwarded ADC clock.
	constant ADC_CLK_EDGE		: std_logic_vector(1 downto 0) := "11";
	-- The maximum search range is 32 (2.5 ns), which guarantees that an edge will be present when Freq > 400 MHz (200 MHz DDR).
	-- Note:  Set the maximum value to 28 (32 - 4) to accomodate half of the 9-tap filter used for edge detection.
	constant ADC_MAX_SEARCH		: integer := 28;
	-- The clocks can be separated by 5 ns (2.5 ns DDR), which is the entire search window range.
	constant ADC_MAX_SEPARATION	: integer := 32;
	-- Do not reset the delay values if the search completes without finding a solution.
	constant ADC_RESET_IF_NOSOL	: boolean := false;
	-- Do not offset the computed solution.
	constant ADC_FINAL_OFFSET	: integer := 0;

end model_specific_package;

package body model_specific_package is

	-- Convert zero to one for the channel count index.
	function zero_to_one (ch_cnt : integer) return integer is
	begin
		if ch_cnt = 0 then
			return 1;
		else
			return ch_cnt;
		end if;
	end zero_to_one;

end model_specific_package;




