--*******************************************************************************************************************************
--
-- Description: This model measures PCI throughput and maximum latency performance.
--					
-- Rev History: 5-28-2014:	Inital release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all;

use work.model_specific_package.all;
use work.dma_package.all;
use work.transceiver_package.all;

entity pci_performance is
port (
    -- Clocks and control.
	rst 			: in std_logic;		-- Active high asynchronous reset.
	clk 			: in std_logic;		-- System clock.

    -- DMA events and status.
	dma_wr			: in std_logic;						-- Logic one indicates that the PLDA_EZDMA2 core is supplying data from a DMA read request.
	dma_wrchannel	: in std_logic_vector(7 downto 0);	-- Logic one indicates that data is available on the designated DMA channel.
	dma_rd			: in std_logic;						-- Logic one indicates that the PLDA_EZDMA2 core is consuming data from a DMA write request.
	dma_rdchannel	: in std_logic_vector(7 downto 0);	-- Logic one indicates that data is needed on the designated DMA channel.
	dma_control		: in DMA_TYPE_6;					-- DMA control settings for the PLDA EZDMA2 core.

    -- Performance measurement registers.
	pci_measurement	: in std_logic_vector(1 downto 0);	-- Logic one clears the PCI throughput (0) or latency (1) register.
	pci_throughput	: out std_logic_vector(63 downto 0);-- Report the measured PCI DMA throughput in bytes/sec (transmitter=63:32, receiver=31:0).
	pci_latency		: out std_logic_vector(63 downto 0)	-- Report the maximum PCI DMA latency in 4 ns clock periods (transmitter=63:32, receiver=31:0).
);
end pci_performance;

architecture structural of pci_performance is

    signal qtr_second		: std_logic_vector(31 downto 0);	-- One second counter based on the 250 MHz PCI clock.
	signal qtr_second_mark	: std_logic;						-- Mark to indicate the completion of a one second timer interval.
	signal transactions		: std_logic_vector(63 downto 0);	-- Transaction count on the transmitter (63:32) and receiver (31:0).
	signal latency			: std_logic_vector(63 downto 0);	-- Latency timer on the transmitter (63:32) and receiver (31:0).
	signal dma_rx			: std_logic;						-- Logic one indicates an active DMA write transaction.
	signal dma_tx			: std_logic;						-- Logic one indicates an active DMA read transaction.
	signal clear_throughput	: std_logic;						-- Single period throughput clear.
    signal latency_timer	: CH_TYPE_32;						-- Counter to time latency response based on the 250 MHz PCI clock.
	signal latency_enable	: CH_TYPE_1;						-- Logic one enable the DMA latency counter.
	signal latency_update	: unsigned(7 downto 0);				-- Logic one indicates that a new maximum value was measured on that channel.
	signal latency_clear	: CH_TYPE_1;						-- Edge detect the PCI measurement enable.
	
begin

--************************************************** THROUGHPUT MEASUREMENTS ***************************************************

	-- Edge detect the throughput clear signal to produce a single period clear.
	SRAM_WR : entity work.clock_crossing
	port map (
		rst					=> rst,
		source_clk			=> clk,
		target_clk  		=> clk,
		source_to_target	=> pci_measurement(0),
		target_enable		=> clear_throughput
	);
	
	process (clk,rst)
	begin
		if rst='1' then
			pci_throughput <= (others => '0');
			qtr_second_mark <= '0';
		
		elsif rising_edge(clk) then
			-- Clear the counters and throughput registers when commanded.
			if clear_throughput = '1' then
				qtr_second_mark <= '1';
				pci_throughput <= (others => '0');
				
			-- Report the performance in units of bytes/sec every quarter second (250 x 10^6 / 4 clock periods).
			elsif qtr_second = x"03B9ACA0" then
				-- Clear the counters on quarter second intervals.
				qtr_second_mark <= '1';
				-- There are eight bytes transferred per count (x8) and the measurement is made in a quarter second interval (x4).
				if AXI_WIDTH = 64 then
					-- Save the current receiver (DMA write) transaction count.
					pci_throughput(31 downto 0) <= transactions(26 downto 0) & "00000";
					-- Save the current tranmitter (DMA read) transaction count.
					pci_throughput(63 downto 32) <= transactions(58 downto 32) & "00000";
				-- There are sixteen bytes transferred per count (x16) and the measurement is made in a quarter second interval (x4).
				else
					-- Save the current receiver (DMA write) transaction count.
					pci_throughput(31 downto 0) <= transactions(25 downto 0) & "000000";
					-- Save the current tranmitter (DMA read) transaction count.
					pci_throughput(63 downto 32) <= transactions(57 downto 32) & "000000";
				end if;
			else
				qtr_second_mark <= '0';
			end if;
		end if;
	end process;

    -- Qualify each DMA transaction and do not count read completions on channel zero.
    dma_rx <= dma_rd and not(dma_rdchannel(0));
    dma_tx <= dma_wr and not(dma_wrchannel(0));
   
	-- Count a one second interval based on the 250 MHz PCI clock.
	QTR_SECOND_CNT : entity work.counter_32
	port map (
		clk		=> clk,
		sclr	=> qtr_second_mark,
		ce		=> '1',
		load	=> '0',
		l		=> (others => '0'),
		q		=> qtr_second
	);

	-- Count the number of receiver transactions over a one second interval.
	RX_TRANSACTIONS : entity work.counter_32
	port map (
		clk		=> clk,
		sclr	=> qtr_second_mark,
		ce		=> dma_rx,
		load	=> '0',
		l		=> (others => '0'),
		q		=> transactions(31 downto 0)
	);

	-- Count the number of transmitter transactions over a one second interval.
	TX_TRANSACTIONS : entity work.counter_32
	port map (
		clk		=> clk,
		sclr	=> qtr_second_mark,
		ce		=> dma_tx,
		load	=> '0',
		l		=> (others => '0'),
		q		=> transactions(63 downto 32)
	);
	
--**************************************************** LATENCY MEASUREMENTS ****************************************************

	-- Perform a measurement on each receiver channel.
	RX_LATENCY : if RX_CHANNEL_COUNT > 0 generate
		-- Create a timer for each receiver channel instantiated.
		RX_DMA : for i in 1 to zero_to_one(RX_CHANNEL_COUNT) generate
			process (clk,rst)
			begin
				if rst='1' then
					latency_enable(i) <= '0';
					latency_clear(i) <= '0';
					latency_update(i-1) <= '0';
		
				elsif rising_edge(clk) then
					-- Start the timer at zero when a DMA request is issued to the PLDA EZDMA2 core.
					if dma_control(i) /= "000000" then
						latency_enable(i) <= '1';
						latency_clear(i) <= '1';
					-- Stop the timer when the first DMA transfer is detected on the channel.
					elsif dma_rd = '1' and dma_rdchannel(i) = '1' and latency_enable(i) = '1' then
						latency_enable(i) <= '0';
						-- Record the latency value if it is the largest measured.
						if latency_timer(i) > latency(31 downto 0) then
							latency_update(i-1) <= '1';
						end if;
					-- Release the clear and suspend updates while timing the event.
					else
						latency_clear(i) <= '0';
						latency_update(i-1) <= '0';
					end if;
				end if;
			end process;
			
			-- Count the number of 250 MHz PCI clock periods between a DMA request and the first transaction.
			RX_LATENCY_CNT : entity work.counter_32
			port map (
				clk		=> clk,
				sclr	=> latency_clear(i),
				ce		=> latency_enable(i),
				load	=> '0',
				l		=> (others => '0'),
				q		=> latency_timer(i)
			);

		end generate;
	end generate;

	-- Perform a measurement on each transmitter channel.
	TX_LATENCY : if TX_CHANNEL_COUNT > 0 generate
		-- Create a timer for each transmitter channel instantiated.
		TX_DMA : for i in RX_CHANNEL_COUNT+1 to TX_CHANNEL_COUNT+RX_CHANNEL_COUNT generate
			process (clk,rst)
			begin
				if rst='1' then
					latency_enable(i) <= '0';
					latency_clear(i) <= '0';
					latency_update(i-1) <= '0';
		
				elsif rising_edge(clk) then
					-- Start the timer at zero when a DMA request is issued to the PLDA EZDMA2 core.
					if dma_control(i) /= "000000" then
						latency_enable(i) <= '1';
						latency_clear(i) <= '1';
					-- Stop the timer when the first DMA transfer is detected on the channel.
					elsif dma_wr = '1' and dma_wrchannel(i) = '1' and latency_enable(i) = '1' then
						latency_enable(i) <= '0';
						-- Record the latency value if it is the largest measured.
						if latency_timer(i) > latency(63 downto 32) then
							latency_update(i-1) <= '1';
						end if;
					-- Release the clear and suspend updates while timing the event.
					else
						latency_clear(i) <= '0';
						latency_update(i-1) <= '0';
					end if;
				end if;
			end process;
			
			-- Count the number of 250 MHz PCI clock periods between a DMA request and the first transaction.
			TX_LATENCY_CNT : entity work.counter_32
			port map (
				clk		=> clk,
				sclr	=> latency_clear(i),
				ce		=> latency_enable(i),
				load	=> '0',
				l		=> (others => '0'),
				q		=> latency_timer(i)
			);

		end generate;
	end generate;

    -- Tie off latency update bits for unused channels.
	NO_CH : if TX_CHANNEL_COUNT+RX_CHANNEL_COUNT < 8 generate
		NO_UPDATE : for i in TX_CHANNEL_COUNT+RX_CHANNEL_COUNT+1 to 8 generate
			latency_update(i-1) <= '0';
		end generate;
	end generate;

	process (clk,rst)
	begin
		if rst='1' then
			latency <= (others => '0');
		
		elsif rising_edge(clk) then
			-- Reset the recorded latency to begin a new measurement.
			if pci_measurement(1) = '1' then
				latency <= (others => '0');
			-- Update the recorded latency if a new measurement is larger than the stored value.
			else
				-- Only one DMA channel is active at any time, so there can never be more than one update pending.
				if to_integer(latency_update) /= 0 then
					-- Update the maximum transmitter latency.
					if to_integer(latency_update) > RX_CHANNEL_COUNT then
						latency(63 downto 32) <= latency_timer(to_integer(latency_update));
					-- Update the maximum receiver latency.
					else
						latency(31 downto 0) <= latency_timer(to_integer(latency_update));
					end if;
				end if;
			end if;
		end if;
	end process;
		
	-- Assign the latency register value to the output port.
	pci_latency <= latency;

end structural;
